unit frmMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  OoMisc, AdPort, StdCtrls, AdStatLt, Mask, JvExMask, JvSpin,
  ExtCtrls, JvComponentBase, JvThreadTimer;

type
  TfmMain = class(TForm)
    ApdComPort1: TApdComPort;
    Label1: TLabel;
    lblComNumber: TLabel;
    Label2: TLabel;
    lblStatus: TLabel;
    ledStatus: TApdStatusLight;
    btnStart: TButton;
    btnStop: TButton;
    edFreq: TJvSpinEdit;
    Timer1: TTimer;
    ledRTS: TApdStatusLight;
    Label3: TLabel;
    ledDTR: TApdStatusLight;
    Label4: TLabel;
    JvThreadTimer1: TJvThreadTimer;
    cbComViewStates: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure btnStartClick(Sender: TObject);
    procedure edFreqChange(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure JvThreadTimer1Timer(Sender: TObject);
  private
    procedure App_Init;
    procedure App_Ini_Gen;
    procedure App_InitCom;
    procedure App_GuiRefresh;
    procedure Gen_Start;
    procedure Gen_Stop;
    procedure Gen_FreqSet(iVal: extended);
    procedure Com_Close;
    procedure Com_Open;
    procedure Com_SetOpened(bOpen: boolean);
    procedure Com_MonitorSetEnabled(bState: boolean);
    { Dclarations prives }
  public
    { Dclarations publiques }
  end;

var
  fmMain: TfmMain;
  bComOK: boolean;
  bComOpened: boolean;
  bGenInProgress: boolean;
  bFreqChangeInProgress: boolean;
  iFreqCount: integer;

implementation

{$R *.DFM}

procedure Delay(ms: cardinal);
var
  T0, T1: cardinal;
begin
  T0 := GetTickCount;
  repeat
    T0 := GetTickCount;
    Application.ProcessMessages;
  until ((T1 - T0) >= ms);
end;

procedure TfmMain.FormCreate(Sender: TObject);
begin
  App_Init;
end;

procedure TfmMain.App_Init;
begin
  App_Ini_Gen;
  App_InitCom;
  Gen_Start;
end;

procedure TfmMain.App_Ini_Gen;
begin
  JvThreadTimer1.Enabled := false;
end;

procedure TfmMain.App_InitCom;
var
  i: integer;
begin
  // search a valid com port. First found is the one used
  for i := 1 to 16 do
  begin
    ApdComPort1.Open := false;
    ApdComPort1.ComNumber := i;
    try
      ApdComPort1.Open := true;
      bComOK := true;
    except
      bComOK := false;
    end;
    if bComOK then break;
  end;
  // one valid com port found - all is OK
  if bComOK then
  begin
    lblComNumber.Caption := format('Com %d', [i]);
  end
  // no valid com port found - no test can be done !
  else begin
    lblComNumber.Caption := 'Aucun port valide trouv !';
    MessageDlg('Impossible d''ouvrir un port com sur ce PC.', mtError, [mbOK], 0);
  end;
end;

procedure TfmMain.App_GuiRefresh;
begin
  // refresh main GUI user controls
  Timer1.Enabled := bGenInProgress;
  {
  edFreq.Enabled := (not bGenInProgress);
  }
  btnStart.Enabled := (not bGenInProgress);
  btnStop.Enabled := bGenInProgress;
  ledStatus.Lit := bGenInProgress;
  if bGenInProgress then
    lblStatus.Caption := 'Activ'
  else
    lblStatus.Caption := 'Dsactiv';
end;

procedure TfmMain.JvThreadTimer1Timer(Sender: TObject);
begin
  if not bGenInProgress then exit;
  if bFreqChangeInProgress then exit;
  // do the job
  ApdComPort1.DTR := not ApdComPort1.DTR;
  ApdComPort1.RTS := ApdComPort1.DTR;
  {
  or if we want DTR and RTS with phase opposed :
  ApdComPort1.RTS := not ApdComPort1.DTR;
  }
  if cbComViewStates.checked then
  begin
    LedDTR.lit := ApdComPort1.DTR;
    LedRTS.lit := ApdComPort1.RTS;
  end;
  Application.ProcessMessages;
end;

procedure TfmMain.Timer1Timer(Sender: TObject);
begin
  // just one visual blink indicator to show when Gen is working
  ledStatus.Lit := not ledStatus.Lit;
end;

procedure TfmMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if bGenInProgress then
    btnStartClick(btnStop);
end;

procedure TfmMain.btnStartClick(Sender: TObject);
begin
  // btnStartClick event can be called by btnStart or by btnStop
  if sender = btnStart then
    Gen_Start
  else if sender = btnStop then
    Gen_Stop;
end;

procedure TfmMain.Gen_Start;
begin
  if not bComOK then
  begin
    MessageDlg('Aucun port valide trouv !', mtError, [mbOK], 0);
    exit;
  end;
  Com_Open;
  JvThreadTimer1.Enabled := true;
  bGenInProgress := true;
  App_GuiRefresh;
end;

procedure TfmMain.Gen_Stop;
begin
  JvThreadTimer1.Enabled := false;
  Com_Close;
  ledStatus.Lit := false;
  bGenInProgress := false;
  App_GuiRefresh;
end;

procedure TfmMain.edFreqChange(Sender: TObject);
begin
  Gen_FreqSet(EdFreq.Value);
end;

procedure TfmMain.Gen_FreqSet(iVal: extended);
begin
  // HP timer events occur at spcified frequency
  // so set real freq value to freq * 2
  // example : 100 periods change for 50 Hz value
  bFreqChangeInProgress := true;
  with JvThreadTimer1 do
  begin
    Enabled := false;
    Delay(100);
    Interval := trunc(1000 / (edFreq.Value * 2));
    Delay(100);
    Enabled := true;
  end;
  bFreqChangeInProgress := false;
end;

procedure TfmMain.Com_SetOpened(bOpen: boolean);
var
  bOK: boolean;
begin
  with ApdComPort1 do
  begin
    try
      ApdComPort1.Open := bOpen;
      bOK := true;
    except
      bOK := false;
    end;
    Delay(100);
    bComOpened := (bOpen and bOK);
  end;
end;

procedure TfmMain.Com_Open;
begin
  Com_SetOpened(true);
  Com_MonitorSetEnabled(true);
end;

procedure TfmMain.Com_Close;
begin
  Com_MonitorSetEnabled(false);
  Com_SetOpened(false);
end;

procedure TfmMain.Com_MonitorSetEnabled(bState: boolean);
begin
  with ApdComPort1 do
  begin
    try

    except

    end;
  end;
  Delay(100);
end;


end.
